#ifndef UOSADC_H
#define UOSADC_H
/*************************************************************************
Analog-Digital-Wandler des ATMega16
20061029  KB  UniOs
ReadADC(Kanal) liest einen Wert vom spezifizierten Kanal (0..7)
Verwendet die interne Referenzspannung - Ergebnis in mV
*************************************************************************/

#include <avr/io.h>

uint16_t ReadADC(uint8_t mux)
{
  uint8_t i;
  uint16_t result = 0;          //Initialisieren wichtig, da lokale Variable
                                //sonst zufllige Werte haben.
  ADCSRA =  _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1); // Frequenzvorteiler auf 64
                                                 // ADC aktivieren (ADEN=1)
  ADMUX = mux | _BV(REFS1) | _BV(REFS0); //Kanal waehlen, interne Referenzspannung

  ADCSRA |= _BV(ADSC);          //eine ADC-Wandlung ("Dummy-Readout" zum "Aufwrmen")
  loop_until_bit_is_clear(ADCSRA,ADSC);   //auf Abschluss der Konvertierung warten

  // Eigentliche Messung - Mittelwert aus 10 aufeinanderfolgenden Wandlungen
  for(i=0;i<10;i++)
  {
    ADCSRA |= _BV(ADSC);        //eine ADC-Wandlung
    loop_until_bit_is_clear(ADCSRA,ADSC); //auf Abschluss der Konvertierung warten
    result += ADCW;             //Beide ADC-Bytes lesen 
  }
  ADCSRA &= ~_BV(ADEN);         //ADC deaktivieren (ADEN=0)

  return result>>2;             //Ergebnis in mV (Referenzspannung ist 2.56 V)
}
#endif // UOSADC_H
